//=============================================================================
// ANNAMutator. (C) 2012 "Dei" & Nobody. Edits by Cozmo, 2020
//
//  Mutator which is responisble for the replacing of all required Actors in
// the map at the game begin *and* while summoning.
//=============================================================================
class ANNAMutator extends CBPMutator;

//#exec OBJ LOAD FILE="..\System\DXWeapon2.u"
//#exec OBJ LOAD FILE="..\System\ANNAExt.u"

var class<Inventory> toReplace[30]; //list of clases that the replacer is supposed to look for
var class<Inventory> replaceTo[30]; //list of classes that the replacer is supposed to replace with
var CDXGameMode AGM;

/**
  Modified, so it just accepts one mutator of that class in the mutator list.
*/
function AddMutator(Mutator M)
{
    if(M.Class != class)
      super.AddMutator(M);
}

function PostBeginPlay()
{
	local UdpServerQuery UdpQuery;

    Super.PostBeginPlay();
}

/**
  Checks the Actor which is given by the parameter <i>in</i> and eventually replaces it.
  Since the parameter is declared with the <b>out</b> modifier, the changing of the parameter
  <i>in</i> will directly effect the Actor given by the <i>in</i> parameter.
*/
function ReplaceMapItem (out Actor in, Class<Actor> inClass)
{
    local Inventory inv, weapon;
    local int i;
    local int length;
    local bool bFound;

    if(Inventory(in) != none)
    {
        for(i=0;  i < ArrayCount(toReplace) ; i++)
        {
            if(inClass == toReplace[i])
            {
                bFound=true;
                break;
            }
        }
		
        if(bFound)
        {
            inv=Spawn(replaceTO[i],in.owner,in.tag,in.location,in.Rotation);
			
            if(inv != none)
            {
                inv.SetPhysics(in.Physics);
                inv.RespawnTime=Inventory(in).RespawnTime;
                in.Destroy();
                in = inv;
            }
        }
    }
	
    Super.ReplaceMapItem (in, inClass);
    
    if(in.isA('AmmoCrate') || in.isA('Inventory') || in.isA('Robot') || in.isA('DeusExDecoration')) // Rightfully stolen from miniMTL.. should decrease network load?
    {
        in.bAlwaysRelevant = False;
		
        if(in.isA('Weapon'))
        {
            weapon = Weapon(in);
			
            if(weapon != None)
            {
                if(!weapon.bHeldItem && !weapon.bTossedOut && weapon.Owner == None) // Make sure we only change it for weapons that are on the wall, and not (previously) owned by anyone.
                {
                    weapon.NetUpdateFrequency = 5.00; // Only send 5 updates per second, instead of default 100.
                }
            }
        }
    }
}

/**
  Right spawns <i>ReplaceMapItem()</i>, because the right summoned Actor can be replaced like all required Actors at the begin too.
*/
function SpawnNotification (out Actor in, Class<Actor> inClass)
{
    ReplaceMapItem(in, inClass); //Cozmo: wait, doesn't this mean it's doing everything in this function any time *anything* spawns??? optimisation todo: 
    Super.SpawnNotification(in, inClass);
}

/*
function bool CheckReplacement(actor Other, out byte bSuperRelevant)
{
	return !ReplaceWith(Other, string(other.Class));
}*/

DefaultProperties
{
}
