//=========================================================================
// CDXGameMode - by Cozmo, 2018
//
// An easy way to make game modes. Custom gametypes will hopefully never
// be made obsolete again.
//=========================================================================
class CDXGameMode extends Mutator config; //info; //abstract;

//#exec OBJ LOAD FILE="..\System\DXWeapon2.u"

var DeusExMPGame gameClass;
var bool bTeamGame; //is this gamemode a team-based one?
var string UNATCOName; //for custom modes (todo)
var string NSFName;
var string testString1;
var int fragLimit;
var float gameDuration;
var bool bGameInProgress;
var DeusExPlayer hurtPlayer;
var int punishNum;

var config bool testBool;

const TEAM_UNATCO	= 0;
const TEAM_NSF 		= 1;
const TEAM_DRAW		= 2;
const TEAM_NONE		= 3;
const TEAM_AUTO	 	= 128;

replication
{
	reliable if(ROLE == ROLE_Authority) //server to client
		gameDuration; //fragLimit //SetObjectiveStr; //GetObjectiveStr, 
	
//	reliable if (Role < ROLE_Authority) //client to server
}

function BeginPlay()
{
	local CDXGameMode agm2;
	
	Super.BeginPlay();
	
	if(Class != Class'CDXGameMode')
	{
		//we must be an AGM child class! Kill the parent
		ForEach AllActors(Class'CDXGameMode', agm2)
		{
			if(agm2 != Self)
				agm2.Destroy();
		}
		
		Level.Game.Event = 'newagm';
		Level.Game.Trigger(self, none);
	}
	
	gameClass = DeusExMPGame(level.Game);
	gameDuration = gameClass.ScoreToWin;
	FixMovers();
	BeginMatch();
	
	//maybe bTeamGame should be declared beforehand?
	if(TeamDMGame(gameClass) != None)
		bTeamGame = true;
	else
		bTeamGame = false;
		
	SetTimer(2.0, false);
}

function PreBeginPlay()
{
	Level.Game.BaseMutator.AddMutator(Self);
}

//function PostNetBeginPlay() //called on clients after joining

function BeginMatch()
{
	local actor a;
	
	if(gameClass.InitialAugs == 0 && gameClass.AugsPerKill == 0)
	{
		//0A detected - delete GEP, LAW, Plasma, Shells, WP //todo: move this to new wep replacement code 
		foreach AllActors(class'Actor', a)
		{
			if(a.IsA('WeaponGEPGun') || a.IsA('WeaponPlasmaRifle') || a.IsA('WeaponLAW') || a.IsA('Ammo20mm') || a.IsA('AmmoRocketWP') || a.IsA('CDXWeaponGEPGun') || a.IsA('CDXWeaponPlasmaRifle') )
			{
				a.Destroy();
			}
		}
	}
	
	//some very basic examples:
	
	/* ATDM
	gameClass.SkillsTotal=2000;
	gameClass.SkillsAvail=2000;
	gameClass.SkillsPerKill=2000;
	gameClass.InitialAugs=2;
	gameClass.AugsPerKill=2;
	gameClass.MPSkillStartLevel=1;
	ResetScores();
	*/
	
	/* BTDM
	gameClass.SkillsTotal=0;
	gameClass.SkillsAvail=0;
	gameClass.SkillsPerKill=0;
	gameClass.InitialAugs=9;
	gameClass.AugsPerKill=0;
	gameClass.MPSkillStartLevel=3;
	ResetScores();
	*/
	
	/* 0A
	gameClass.SkillsTotal=0;
	gameClass.SkillsAvail=0;
	gameClass.SkillsPerKill=0;
	gameClass.InitialAugs=0;
	gameClass.AugsPerKill=0;
	gameClass.MPSkillStartLevel=3;
	//delete weps here
	ResetScores();
	*/
}

function FixMovers()
{
	local int oldKN;
	local mover mov;

	//Lexichu's version!
	foreach AllActors(class'Mover', mov)
	{
		if(mov.MoverEncroachType == ME_StopWhenEncroach)
		{
			oldKN = -1;
			
			if(mov.KeyNum != 0)
			{
				oldKN = mov.KeyNum;
				mov.KeyNum = 0;
			}
			
			mov.MoverEncroachType = ME_IgnoreWhenEncroach;
			
			if(oldKN != -1)
				mov.KeyNum = oldKN;
		}
	}
}

// ---------------------------------------------------------------------------------------
// Stuff that happens during gameplay
// ---------------------------------------------------------------------------------------
function Tick(float deltaTime)
{
	local int i;
	
	//all gamemode timing (Level.TimeSeconds) here! Timer() sucks, don't use it here
	CheckForEndOfGame();
	
	Super.Tick(deltaTime);
}

function CDXExtensionCommand(DeusExPlayer player, string playerCommand, out int CmdNotFound) //custom functions. Example: "say /dropflag", "say /taunt 1" etc
{	
	local mpnsf mpn;
	local float argTime;
    local string currentMap;
	local int i, playerID;
	local DeusExPlayer dxp;
	local CDXGameMode agm2;
	local string CDXVersion;
	
	//admin commands here:
	if(player.bAdmin == true)
	{
		currentMap = left(string(level),instr(string(level),"."));
		CDXVersion = "140";
	
		if(playerCommand ~= "stopgame")
		{
			StopMatch("Game Over", TEAM_NONE, "An admin forcefully ended the game!");
			return;
		}
		else if(mid(playerCommand, 0, 5) ~= "time ") //"cc time 5" would set game duration to 5 mins
		{
			argTime = float(mid(playerCommand, 5));
			gameDuration = argTime + (Level.TimeSeconds / 60);
			broadcastmessage("Game time set to "$int(argTime)$" minutes.");
			return;
			//possibly save it in config?
		}
		else if(playerCommand == "lockdown")
		{
			if(gameClass.bNoMonsters == true)
			{
				gameClass.bNoMonsters = false;
				BroadCastMessage("Server is no longer under lockdown");
			}
			else
			{
				gameClass.bNoMonsters = true;
				BroadCastMessage("Server is now under lockdown");
			}
			
			return;
		}
		else if(playerCommand ~= "atdm")
		{
			gameClass.SkillsTotal=2000;
			gameClass.SkillsAvail=2000;
			gameClass.SkillsPerKill=2000;
			gameClass.InitialAugs=2;
			gameClass.AugsPerKill=2;
			gameClass.MPSkillStartLevel=1;
			gameClass.SaveConfig();
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_Team");
			return;
		}
		else if(playerCommand ~= "adm")
		{
			gameClass.SkillsTotal=2000;
			gameClass.SkillsAvail=2000;
			gameClass.SkillsPerKill=2000;
			gameClass.InitialAugs=2;
			gameClass.AugsPerKill=2;
			gameClass.MPSkillStartLevel=1;
			gameClass.SaveConfig();
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_DeathMatch");
			return;
		}
		else if(playerCommand ~= "0a")
		{
			gameClass.SkillsTotal=0;
			gameClass.SkillsAvail=0;
			gameClass.SkillsPerKill=0;
			gameClass.InitialAugs=0;
			gameClass.AugsPerKill=0;
			gameClass.MPSkillStartLevel=3;
			gameClass.SaveConfig();
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_Team");
			return;
		}
		else if(playerCommand ~= "0adm")
		{
			gameClass.SkillsTotal=0;
			gameClass.SkillsAvail=0;
			gameClass.SkillsPerKill=0;
			gameClass.InitialAugs=0;
			gameClass.AugsPerKill=0;
			gameClass.MPSkillStartLevel=3;
			gameClass.SaveConfig();
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_DeathMatch");
			return;
		}
		else if(playerCommand ~= "btdm")
		{
			gameClass.SkillsTotal=0;
			gameClass.SkillsAvail=0;
			gameClass.SkillsPerKill=0;
			gameClass.InitialAugs=9;
			gameClass.AugsPerKill=0;
			gameClass.MPSkillStartLevel=3;
			gameClass.SaveConfig();
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_Team");
			return;
		}
		else if(playerCommand ~= "bdm")
		{
			gameClass.SkillsTotal=0;
			gameClass.SkillsAvail=0;
			gameClass.SkillsPerKill=0;
			gameClass.InitialAugs=9;
			gameClass.AugsPerKill=0;
			gameClass.MPSkillStartLevel=3;
			gameClass.SaveConfig();
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_DeathMatch");
			return;
		}
		else if(playerCommand ~= "kdm")
		{
			//for some reason, doing this without commands doesn't work
			ConsoleCommand("set DeusExMPGame SkillsTotal 3000");
			ConsoleCommand("set DeusExMPGame SkillsAvail 3000");
			ConsoleCommand("set DeusExMPGame SkillsPerKill 1000");
			ConsoleCommand("set DeusExMPGame InitialAugs 3");
			ConsoleCommand("set DeusExMPGame AugsPerKill 1");
			ConsoleCommand("set DeusExMPGame MPSkillStartLevel 1");
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_DeathMatch");
			return;
		}
		else if(playerCommand ~= "ktdm")
		{
			//for some reason, doing this without commands doesn't work
			ConsoleCommand("set DeusExMPGame SkillsTotal 3000");
			ConsoleCommand("set DeusExMPGame SkillsAvail 3000");
			ConsoleCommand("set DeusExMPGame SkillsPerKill 1000");
			ConsoleCommand("set DeusExMPGame InitialAugs 3");
			ConsoleCommand("set DeusExMPGame AugsPerKill 1");
			ConsoleCommand("set DeusExMPGame MPSkillStartLevel 1");
			ConsoleCommand("servertravel"@currentMap$"?Game=CDX_"$CDXVersion$".CDX_Team");
			return;
		}
		
		//testing commands:
		else if(playerCommand ~= "sendmsg")
		{
			NotifyPlayers("a");
			return;
		}
		else if(playerCommand ~= "debug1")
		{
			broadcastmessage(class);
			
			if(class == Class'CDXGameMode')
				broadcastmessage("agm detected. No replace.");
				
			ForEach AllActors(Class'CDXGameMode', agm2)
				broadcastmessage(agm2);
				
			//SetObjectiveStr("debug objective here!", player);	
			
			/*			//oh, it turns out it is this easy to write/read ini during runtime
			broadcastmessage(testBool);
			testBool = false;
			SaveConfig();
			return;	
			*/
		
			//broadcastmessage(tag); //it turns out: you CAN use caps, numbers and some symbols (-) in name vars. Maybe not spaces?
			
			//broadcastmessage(DeusExRootWindow(player.rootwindow).hud.augDisplay.name);
			
			//if(player.weapon != none)
			//	broadcastmessage(player.weapon.GetStateName());
		}
		else if(playerCommand ~= "debug2")
		{
			broadcastmessage(Class);
		}
		else if(playerCommand ~= "testbot")
		{
			mpn = Spawn(class'MPNSF',,,player.Location + (player.CollisionRadius+Class'MPNSF'.Default.CollisionRadius+30) * Vector(player.Rotation) + vect(0,0,1) * 15 );
			mpn.PlayerReplicationInfo = Spawn(Class'PlayerReplicationInfo');
			mpn.PlayerReplicationInfo.Team = 1;
			mpn.PlayerReplicationInfo.PlayerName = "Player Bot 500";
			return;
		}
	//	else if(playerCommand == "test1")
	//	{
	//		Spawn(class'CDXMapList');
	//	}
	}
	
	//regular commands:
	
	
	
	//if we reach here, we didn't find a command
	CmdNotFound = 1;
}


//--------------------------------------------------------------------------
// Command Hooks
//
// This allows you to run your own code when player functions are called.
// You can return true to prevent the original from calling
//--------------------------------------------------------------------------
function bool Say(DeusExPlayer sayer, string sayMsg)
{
	return false; //return true to prevent normal CDX Say()
}

function bool TeamSay(DeusExPlayer sayer, string sayMsg)
{
	return false; //return true to prevent normal CDX TeamSay()
}

function bool PawnFragged(Pawn killer, optional Pawn victim, optional name damageType)
{
	return false; //return true to prevent normal frag code
}

function bool DropItem(Pawn player, Inventory inv)
{
	return false; //return true to prevent dropping inventory items
}

function bool SpectateON(Pawn player)
{
	return false; //return true to prevent CDX spectate
}

function bool SpectateOFF(Pawn player)
{
	return false; //return true to prevent CDX un-spectate
}

function bool TryJump(Pawn player)
{
	return false; //return true to prevent normal jump
}

function bool PlayerLanded(Pawn player)
{
	return false; //return true to prevent normal landing code
}

function bool PlayerSupportsActor(Pawn player, actor standingActor)
{
	return false; //return true to stop old standing-on-me behaviour
}

function bool TrySuicide(Pawn player)
{
	return false; //return true to prevent suicide
}

// -------------------------------------------------------------
// Futureproof hooks. Check "thing", and return whether you
// want it to happen or not (similar to above hooks)
// -------------------------------------------------------------
function bool CanIDoThat(name thing, actor caller, optional int extra1, optional string extra2)
{
	return true; //return false to prevent thing from happening
}

simulated function bool CanClientDoThat(name thing, actor caller, optional int extra1, optional string extra2)
{
	return true; //return false to prevent thing from happening
}

// -------------------------------------------------------------
// Other useful functions
// -------------------------------------------------------------
function PlayerJoin(Pawn player)	//this doesn't currently work
{
//	log(DeusExPlayer(player).PlayerReplicationInfo.PlayerName$" joined the game!!");
}

function PlayerLeave(Pawn player)
{
	//called when a player leaves the game (needs testing)
}

function ModifyPlayer(Pawn Other) //affect any player pawn after they're spawned
{
	local Human P;
	
	if(Other.IsA('Human'))
		P = Human(Other);
	
	if(P != None)
	{
		SetObjectiveStr(GetObjectiveStr(), P); //set the player's objective whenever they spawn
		
		//when a player respawns, do stuff here:
	}
}

function ResetScores() //not sure this function is even needed?
{
	local Pawn CurPawn;
	
	for(curPawn = Level.PawnList; curPawn != None; curPawn = curPawn.nextPawn)
	{
		if(curPawn.IsA('DeusExPlayer'))
			Level.Game.RestartPlayer(curPawn);
	}
}

function NotifyPlayers(string notificationMsg)
{
	local Pawn curPawn;
	
	for(curPawn = Level.PawnList; curPawn != None; curPawn = curPawn.nextPawn)
	{
		if(curPawn.IsA('DeusExPlayer'))
		{
		/*
			if (bTimeCondition)
				DeusExPlayer(curPawn).MultiplayerNotifyMsg( DeusExPlayer(curPawn).MPMSG_TimeNearEnd, param, winningStr );
			else
				DeusExPlayer(curPawn).MultiplayerNotifyMsg( DeusExPlayer(curPawn).MPMSG_CloseKills, param, winningStr );*/
				
			DeusExPlayer(curPawn).MultiplayerNotifyMsg(9, 1, "Debug: Sending a test message to all players!");
			//DeusExPlayer(curPawn).MultiplayerNotifyMsg(9, 1, notificationMsg);
		}
	}
}

// ---------------------------------------------------------------------------------
// Main game logic
// ---------------------------------------------------------------------------------
function CheckForEndOfGame(optional Pawn Killer)
{
	local float timeLimit;
	
	if(bGameInProgress == false || ROLE < ROLE_Authority)
		return;
	
	// Note: games with a frag limit should *also* have a time limit! Other modes can remove the time limit if they want
	
	if(fragLimit != 0)
	{
	/*
		if(killer.PRI.Score > fragLimit)
		{
			StopMatch();
		}
		else
		{
			//notifications: 10 kills, 5 kills, 3, 2, 1?
		}
	*/
	}
	
	timeLimit = gameDuration * 60.0;

	if(Level.TimeSeconds >= timeLimit)
	{
		bGameInProgress = false;
		
		if(bTeamGame)
		{
			StopMatch(GetFinishedString(), GetWinningTeam(), ""); //can use the "" for notable K/D, damage, etc
		}
		else
		{
			if(GetWinningPlayer() == None)
				StopMatch("The match ended in a draw!", 2, ""); 
			else
				StopMatch(GetWinningPlayer().PlayerReplicationInfo.PlayerName$" has won the match!", 2, ""); //can use the "" for notable K/D, damage, etc
		}
	}
	else
	{
		//do notifications at: 5m, 2m, 1m, 30s, 10s?
	}
}

function StopMatch(string winnerString, int winningTeam, string additionalInfo)
{
	local Pawn curPawn;
	
	gameClass.PreGameOver();
	
	
	
	for(curPawn = Level.PawnList; curPawn != None; curPawn = curPawn.nextPawn)
	{
		if(curPawn.IsA('DeusExPlayer'))
		{
			DeusExPlayer(curPawn).Event = 'dsoc3'; //test
			DeusExPlayer(curPawn).Trigger(self, curPawn);
	
			DeusExPlayer(curPawn).ShowMultiplayerWin(winnerString, winningTeam, "", "", additionalInfo);
			DeusExPlayer(curPawn).ReducedDamageType = 'All'; 
		}
	}
	
	gameDuration = 0;
	gameClass.GameOver();
}

// ---------------------------------------------------------------------------------
// Scoreboard section
// ---------------------------------------------------------------------------------
function int GetWinningTeam() //might need updating to include bots!
{
	local Pawn curPawn;
	local int scoreNSF, scoreUNATCO;
	
	if(bTeamGame == false)
		return TEAM_NONE;
		
	for(curPawn = Level.PawnList; curPawn != None; curPawn = curPawn.nextPawn) //add total score for each team
	{
		if(curPawn.IsA('DeusExPlayer'))
		{
			if(curPawn.PlayerReplicationInfo != None)
			{
				if(curPawn.PlayerReplicationInfo.Team == TEAM_UNATCO)
					scoreUNATCO += curPawn.PlayerReplicationInfo.Score;
				else
					scoreNSF += curPawn.PlayerReplicationInfo.Score;
					
			}
		}
	}
	
	if(scoreNSF == scoreUNATCO) //if we're drawing, then the team with fewer deaths wins
	{
		for(curPawn = Level.PawnList; curPawn != None; curPawn = curPawn.nextPawn)
		{
			if(curPawn.IsA('DeusExPlayer'))
			{
				if(curPawn.PlayerReplicationInfo != None)
				{
					if(curPawn.PlayerReplicationInfo.Team == TEAM_UNATCO)
						scoreUNATCO -= curPawn.PlayerReplicationInfo.Deaths;
					else
						scoreNSF -= curPawn.PlayerReplicationInfo.Deaths;
						
				}
			}
		}
	}
	
	if(scoreNSF == scoreUNATCO)
		return TEAM_DRAW;
	else if(scoreNSF > scoreUNATCO)
		return TEAM_NSF;
	else
		return TEAM_UNATCO;
	
	//return TEAM_NONE; //use this for custom games if you want!
}

function Pawn GetWinningPlayer()
{
	local Pawn curPawn, curWinner, lastWinner; //if two players have same score in DM, it's a draw
	local int curWinScore, lastWinScore;
	
	for(curPawn = Level.PawnList; curPawn != None; curPawn = curPawn.nextPawn)
	{
		if(curPawn.PlayerReplicationInfo != None)
		{
			if(curPawn.PlayerReplicationInfo.Score >= curWinScore)
			{
				if(curWinner != None)
				{
					lastWinner = curWinner;
					lastWinScore = curWinScore;
				}
					
				curWinScore = curPawn.PlayerReplicationInfo.Score;
				curWinner = curPawn;
			}
		}
	}
	
	if(curWinner == none)
		return none;
		
	if(lastWinner == none)
		return curWinner;
	
	if(curWinScore == lastWinScore) //if we're drawing, check deaths
	{
		curWinScore -= curWinner.PlayerReplicationInfo.Deaths;
		lastWinScore -= lastWinner.PlayerReplicationInfo.Deaths;
		
		if(curWinScore == lastWinScore)
			return none; //TEAM_NONE draw!
		else
			if(lastWinScore > curWinScore)
				curWinner = lastWinner;
	}
	
	return curWinner;
}

function string GetObjectiveStr()
{
	if(bTeamGame)
	{
		if(fragLimit != 0)
			return "First team to reach "$fragLimit$" kills wins!";
		else
			return "The team with the highest kill count wins!";
	}
	else
	{
		if(fragLimit != 0)
			return "First player to reach "$fragLimit$" kills wins!";
		else
			return "The player with the highest kill count wins!";
	}
}

function SetObjectiveStr(string objStr2, optional DeusExPlayer P)
{
	local Pawn curPawn;

	if(P == None)
	{		
		for(curPawn = Level.PawnList; curPawn != None; curPawn = curPawn.nextPawn)
		{
			if(curPawn.IsA('DeusExPlayer'))
				SetObjectiveStrPawn(objStr2, curPawn);
		}
	}
	else
		SetObjectiveStrPawn(objStr2, P);
}

function SetObjectiveStrPawn(string objStr2, Pawn P)
{
	local DeusExPlayer dxp;
	
	dxp = DeusExPlayer(P);
	
	//a hack for communicating with CDX
	dxp.strStartMap = objStr2;
	dxp.Event = 'dsoc5';
	dxp.Trigger(self, P);
}

function string GetFinishedString()
{
	local int wt;
	
	wt = GetWinningTeam();
	
	if(wt == TEAM_DRAW)
		return "The match ended in a draw!";
	else if(wt == TEAM_NONE)
		return "The game has finished!";
	
	return GetTeamName(wt)$" has won the match!";
}

function string GetTeamName(int teamID) //in case you want custom team names. NOTE: This is only used for endgame string currently!
{
	if(teamID == TEAM_UNATCO)
		return "Team UNATCO";
		
	if(teamID == TEAM_NSF)
		return "Team NSF";
		
	return "Team "$teamID;
}

function string GetGameName() //fills in the GameType field in server browser. Modders, change this to whatever you want
{
	local string str;
	
	//str = "CDX 100 "; //the main point of this was to show CDX version in browser, but there's not much room lol
	str = "CDX ";
	
	if(gameClass != None)
	{
		if(bTeamGame)
		{
			if(gameClass.SkillsTotal==2000 && gameClass.SkillsAvail==2000 && gameClass.SkillsPerKill==2000 && gameClass.InitialAugs==2 && gameClass.AugsPerKill==2 && gameClass.MPSkillStartLevel==1)
				str = str$"ATDM";
			else if(gameClass.SkillsTotal==0 && gameClass.SkillsAvail==0 && gameClass.SkillsPerKill==0 && gameClass.InitialAugs==9 && gameClass.AugsPerKill==0 && gameClass.MPSkillStartLevel==3)
				str = str$"BTDM";
			else if(gameClass.SkillsTotal==0 && gameClass.SkillsAvail==0 && gameClass.SkillsPerKill==0 && gameClass.InitialAugs==0 && gameClass.AugsPerKill==0 && gameClass.MPSkillStartLevel==3)
				str = str$"Zero Aug";
			else
				str = str$"Team Custom";
		}
		else
		{
			if(gameClass.SkillsTotal==2000 && gameClass.SkillsAvail==2000 && gameClass.SkillsPerKill==2000 && gameClass.InitialAugs==2 && gameClass.AugsPerKill==2 && gameClass.MPSkillStartLevel==1)
				str = str$"ADM";
			else if(gameClass.SkillsTotal==0 && gameClass.SkillsAvail==0 && gameClass.SkillsPerKill==0 && gameClass.InitialAugs==9 && gameClass.AugsPerKill==0 && gameClass.MPSkillStartLevel==3)
				str = str$"BDM";
			else if(gameClass.SkillsTotal==0 && gameClass.SkillsAvail==0 && gameClass.SkillsPerKill==0 && gameClass.InitialAugs==0 && gameClass.AugsPerKill==0 && gameClass.MPSkillStartLevel==3)
				str = str$"0A DeathMatch";
			else
				str = str$"DeathMatch";
		}
	}
	
	return str;
}

function Timer() //was PostBeginPlay. Changed for (hopefully) more compatibility with MTL WepSelect
{
	local Inventory inv;
	local Pawn npcI;

	//super.PostBeginPlay();

	foreach AllActors(class'Inventory', inv)
	{
		ReplaceWith(inv, string(inv.class));
	}
	
	inv = None;
}

//Cozmo: borrowed from Lexi's DXW2 (thank you!)
function bool ReplaceWith(Actor in, string inClass)
{
    local Actor act;
    local Inventory inv;
	local bool bSwapped;
	local class<Actor> grabClass;

	inv = Inventory(in);
	if(inv == None)
		return False;

	grabClass = inv.Class;
	
	//Mixture of CDX weps and DXW2 weps
	if(ClassIsChildOf(grabClass, class'DeusEx.WeaponAssaultGun'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponAssaultGun');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponAssaultShotgun'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponAssaultShotgun');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponCombatKnife'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponCombatKnife');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponEMPGrenade'))
		bSwapped = ItemSwapperoni(inv,class'WeaponEMPGrenade2');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponFlamethrower')) 
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponFlamethrowerEXFR');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponGasGrenade'))
		bSwapped = ItemSwapperoni(inv,class'WeaponGasGrenade2');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponGEPGun'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponGEPGun');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponLAM'))
		bSwapped = ItemSwapperoni(inv,class'WeaponLAM2');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponLAW'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponLAW');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponMiniCrossbow'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponMiniCrossbow');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponNanoSword'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponDTS');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponPistol'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponPistol');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponPlasmaRifle'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponPlasmaRifle');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponRifle'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponRifle');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponSawedOffShotgun'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponSawedOff');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponShuriken'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponKnives');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponStealthPistol'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponStealthPistol');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponPepperGun'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponPepperGun');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponSword'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponSword');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponCrowbar'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponCrowbar');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponBaton'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponBaton');
	else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponProd'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponProd');
	//else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponNanoVirusGrenade'))
	//	bSwapped = ItemSwapperoni(inv,class'WeaponNanoVirusGrenade2');
	//else if(ClassIsChildOf(grabClass, class'DeusEx.WeaponHideAGun'))
	//	bSwapped = ItemSwapperoni(inv,class'WeaponHideAGun2');
	else if(ClassIsChildOf(grabClass, class'DeusEx.FireExtinguisher'))
		bSwapped = ItemSwapperoni(inv,class'CDXWeaponFireExt');

	return bSwapped;
}

function bool ItemSwapperoni(inventory toSwap, class<Inventory> swapTo)
{
	local Inventory newItem;
	local Pawn toOwner;

	if(toSwap != None)
	{
		if(toSwap.Owner != None)
			toOwner = Pawn(toSwap.Owner);
			
		newItem = Spawn(swapTo, toSwap.Owner, toSwap.Tag, toSwap.Location, toSwap.Rotation);
		
		if(newItem != None)
		{
			//log("Replacing" @toSwap.Name@ "with" @swapTo, 'W2Spawner');
			if(toOwner != None)
			{
				newItem.InitialState='Idle2';
				newItem.GiveTo(toOwner);
				newItem.SetBase(toOwner);
				if(ClassIsChildOf(swapTo, Class'DeusEx.DeusExWeapon'))
					Weapon(newItem).GiveAmmo(ToOwner);
				newItem.bHeldItem = true;
				newItem.bTossedOut = false;
				ToOwner.SwitchToBestWeapon();
				//log("Gave" @toOwner.Name@ "their new" @swapTo $"!", 'W2Spawner');
			}
			else
			{
				newItem.SetPhysics(toSwap.Physics);
				newItem.RespawnTime = toSwap.RespawnTime;
				newItem.SetCollisionSize(toSwap.CollisionRadius, toSwap.CollisionHeight);
			}
			
			toSwap.Destroy();
			return True;
		}
	}
	
	return False;
}

DefaultProperties
{
	fragLimit=0
	gameDuration=15
	bTeamGame=false
	bGameInProgress=true
	bAlwaysRelevant=true
	RemoteRole=ROLE_SimulatedProxy;
	NetUpdateFrequency=10
}
