//=========================================================================
// CDXProjectile
//=========================================================================
class CDXProjectile extends DeusExProjectile;

simulated function Tick(float deltaTime)
{
	local float dist, size;
	local Rotator dir;
	local vector TargetLocation; 
	local vector vel;
	local vector NormalHeading;
	local vector NormalDesiredHeading;
	local float HeadingDiffDot;
	local vector zerovec;

	if (bStuck)
		return;

	Super.Tick(deltaTime);

	if(VSize(LastSeenLoc) < 1)
		LastSeenLoc = Location + Normal(Vector(Rotation)) * 10000;
		
	if (Role == ROLE_Authority)
		bHasNetworkTarget = (Target != None);
	else
		bHadLocalTarget = (bHadLocalTarget || (Target != None));

	if(bTracking && Target != None)
	{
		// check it's range
		dist = Abs(VSize(Target.Location - Location));
		
		if (dist > MaxRange) // if we're out of range, lose the lock and quit tracking
		{
			bTracking = False;
			Target = None;
			return;
		}
		else
		{
			// get the direction to the target
			if(Level.NetMode == NM_Standalone)
				TargetLocation = Target.Location;
			else
				TargetLocation = AcquireMPTargetLocation();
			
			if (Role == ROLE_Authority)
				NetworkTargetLoc = TargetLocation;
			
			LastSeenLoc = TargetLocation;
			dir = Rotator(TargetLocation - Location);
			dir.Roll = 0;

			if (Level.Netmode != NM_Standalone)
			{
				NormalHeading = Normal(Vector(Rotation));
				NormalDesiredHeading = Normal(TargetLocation - Location);
				HeadingDiffDot = NormalHeading Dot NormalDesiredHeading;
			}

			// set our new rotation
			bRotateToDesired = True;
			DesiredRotation = dir;

			// move us in the new direction that we are facing
			size = VSize(Velocity);
			vel = Normal(Vector(Rotation));

			if (Level.NetMode != NM_Standalone)
				size = FMax(HeadingDiffDot,0.4) * Speed;
				
			Velocity = vel * size;
		}
	}
	else
	{
		// make the rotation match the velocity direction
		SetRotation(Rotator(Velocity));
	}

	dist = Abs(VSize(initLoc - Location));

	if (dist > AccurateRange)		// start descent due to "gravity"
		Acceleration = Region.Zone.ZoneGravity / 2;

	if ((Role < ROLE_Authority) && (bAggressiveExploded))
		Explode(Location, vect(0,0,1));
}

defaultproperties
{
}