//=============================================================================
// ANNATeam. (C) 2012 "Dei" & Nobody. Edit by Cozmo, 2018
//
// ALL team games (BTDM, ATDM, 0A, CTF etc) will exclusively use this gametype
//=============================================================================
class CDX_Team extends MTLTeam;

var string GameTypeName;
var string recentIP[40];
var int suspicion[40];
var float lastConnectTime[40];
var ANNAManager Manager;
var CDXGameMode AGM;
var CDXServerQuery CSQ;
var bool bHasGameFinished;
//var Class<ANNAPlayer> V1F;
//var Class<ANNAPlayer> V1E;
//var String TeamNsfString, TeamUnatcoString, TeamDrawString; //hopefully redeclaring str overrides localisation 
//																					(if I implement custom team names...)

function PostBeginPlay()
{
	Super.PostBeginPlay();
	
	Spawn(class'MutPlayerID');
	
	if(GameReplicationInfo == none)
		GameReplicationInfo = Spawn(GameReplicationInfoClass);
		
	HandleCSQ();
	HandleAGM();
}

function HandleCSQ()
{
	local UdpServerQuery UdpQuery;
	
	if(Level.NetMode != NM_Standalone)
	{
		foreach AllActors(class'UdpServerQuery', UdpQuery)
		{
			if(UdpQuery != none)
				break;
		}    

		if(UdpQuery == none)
		{
			CSQ = Spawn(class'CDXServerQuery');
		}
		else
		{
			if(CDXServerQuery(UdpQuery) == none)
			{
				UdpQuery.Destroy();
				CSQ = Spawn(class'CDXServerQuery');
			}
		}
	}
}

function HandleAGM()
{
	local DeusExMPGame gameClass;
	local CDXGameMode agm2;
	local Pawn P;
	
	foreach AllActors(class'CDXGameMode', agm2) 
		AGM = agm2;
	
	//create an AGM if the map doesn't have one  
	if(AGM == None)
		AGM = Spawn(class'CDXGameMode'); //todo: make this spawn the correct AGM according to maplist or something
	
	//set player AGM
	for(P=Level.PawnList; P!=None; P=P.NextPawn)
	{
		if(P.IsA('ANNAPlayer'))
		{
			ANNAPlayer(P).AGM = AGM;
		}
	}

	//serverquery
	CSQ.AGM = AGM;
	
	//maplist?

	//scoreboard? idk, maybe it's better for AGM to interact with scoreboard indirectly, via GI and and player vars
}

function Trigger(actor Other, pawn EventInstigator )
{
	if(Event == 'newagm')
		HandleAGM();
	
	Event = '';
}

/**
  Sets the MPL depending on the just loaded map.
*/
event InitGame(string Options, out string Error)
{
    local ANNAManager curManager;
    local bool bFound;
    
    Super.InitGame(Options,Error);

    Manager = findManager();
    if(Manager == None)
      Manager = Spawn(class'ANNAManager');

    CBPMutator(level.Game.BaseMutator).AddCBPMutator(Spawn(class'ANNAMutator'));
}

/**
  Find and return the ANNAManager
*/
function ANNAManager findManager()
{
  local ANNAManager curManager;
  local bool bFound;

  if(Manager != None)
    return Manager;

  foreach AllActors(class'ANNAManager', curManager)
    if(curManager != None)
      return curManager;

  return none;
}

function bool CheckVictoryConditions(Pawn Killer, Pawn Killee, String Method)
{
	//This functionality has been moved to CDXGameMode!
	
	return false;
}

function bool CheckIPSpam(string Address)
{
	local int i, j;
	
	// strip port number
	j = InStr(Address, ":");
	
	if(j != -1)
		Address = Left(Address, j);
	
	for(i = 0; i < 40; i ++)
	{
		if(recentIP[i] == Address)
		{
			if(Level.TimeSeconds <= lastConnectTime[i] + 20)
			{
				suspicion[i] ++;
			}
			
			lastConnectTime[i] = Level.TimeSeconds;
			
			if(suspicion[i] >= 4)
			{
				return true;
				Log(Address$" was denied for reconnect spam");
			}
			else
			{
				return false;
			}
		}
	}
	
	//IP wasn't found. Add it
	for(i = 0; i < 40; i ++)
	{
		if(recentIP[i] == "")
		{
			recentIP[i] = Address;
			lastConnectTime[i] = Level.TimeSeconds;
			return false;
		}
	}
	
	return false; //just in case
}

event PreLogin(string Options, string Address, out string Error, out string FailCode)
{
	//local string ns;
	//
	//ns = ParseOption (Options, "natives");
	//
	//if(ns != "true")
	//{
	//	Error = "Please visit PlayDXMP.com to update your client!";
	//	return; //don't allow Engine.PreLogin to run, or it'll set Error to "" again
	//}
	
	// http://aluigi.altervista.org/adv/unrfs-adv.txt
    if(InStr(ParseOption(Options,"Class"),"%")!=-1 )
    {
        Error    = "CVE-2004-1805";
    }
	else if(bNoMonsters == true) //ANNAExt can't access us normally, so we're using an unused engine var :/
	{
		Error = "This server is not currently accepting new connections";
	}
	else
	{
		//check IP for reconnect spam (ban evasion)
		if(CheckIPSpam(Address) == true)
		{
			Error = "Too many reconnects. Please try again later!";
		}
		else
		{
			Super.PreLogin(Options, Address, Error, FailCode);
		}
	}
}

event PlayerPawn Login (string Portal, string Z56, out string Z57, Class<PlayerPawn> SpawnClass)
{
	local Pawn PawnLink;
	local PlayerPawn TestPlayer;
	local ANNAPlayer Z5B;
	local int Z5C;
	local string Z5D;
  
	for(PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.NextPawn)
	{
		TestPlayer = PlayerPawn(PawnLink);
		
		if(TestPlayer != None)
		{
			if(Len(TestPlayer.Password) == 0)
			{
				TestPlayer.Password = class'ANNAManager'.static.generateUID();
			}
		}
	}
	
	//evil MTL stuff begin
	if ( (MaxPlayers > 0) && (NumPlayers >= MaxPlayers) )
	{
		Z57=TooManyPlayers;
		return None;
	}
	Z5C=128;
	if ( HasOption(Z56,"Team") )
	{
		Z5D=ParseOption(Z56,"Team");
		if ( Z5D != "" )
		{
			Z5C=int(Z5D);
		}
	}
	if (  !ApproveTeam(Z5C) )
	{
		Z5C=128;
	}
	if ( Z5C == 128 )
	{
		Z5C=GetAutoTeam();
	}
	if ( Z5C == 1 )
	{
		SpawnClass=V1E;
	} else {
		SpawnClass=V1F;
	}
	ChangeOption(Z56,"Class",string(SpawnClass));
	ChangeOption(Z56,"Team",string(Z5C));
	Z5B=ANNAPlayer(Super(DeusExMPGame).Login(Portal,Z56,Z57,SpawnClass));
	if ( Z5B != None )
	{
		Z5B.V522(Z5B.PlayerReplicationInfo.PlayerName); //changed from v52 to v522
	}
	return Z5B;
	//evil MTL stuff end

	//return Super.Login(Portal, Z56, Z57, SpawnClass);
}


exec function ConsoleKick(int playerID)
{
  if(Manager == None)
    Manager = findManager();
  if(Manager == None)
    Manager = Spawn(class'ANNAManager');
    
  Manager.serverKick(playerID);
}

exec function ConsoleKickBan(int playerID)
{
  if(Manager == None)
    Manager = findManager();
  if(Manager == None)
    Manager = Spawn(class'ANNAManager');
    
  Manager.serverBan(playerID);
}

exec function ConsolePlayerList()
{
  if(Manager == None)
    Manager = findManager();
  if(Manager == None)
    Manager = Spawn(class'ANNAManager');
    
  Manager.serverPlayerList();
}

event PostLogin(playerpawn NewPlayer)
{
	local ANNAPlayer DXPlayer;
	local float timeLimit;

	Super.PostLogin(NewPlayer);

	DXPlayer = ANNAPlayer(NewPlayer);
	DXPlayer.AGM = AGM;
   
	if(DXPlayer != None)
	{
		if(Manager != None && Manager.bResetSkin)
			DXPlayer.CheckSkin();

		DXPlayer.CheckInventory();
		AGM.PlayerJoin(DXPlayer);

		timeLimit = AGM.gameDuration * 60.0;

		if(Level.TimeSeconds >= timeLimit)
			DXPlayer.ReducedDamageType = 'All';
	}
}

function Killed(Pawn VC7, Pawn Z58, name VC8)
{
	local bool NotifyDeath;
	local DeusExPlayer otherPlayer;
	local Pawn CurPawn;
	local pawn other; //I cba anymore
	local pawn killer;
	local name damageType;
	
	//function Killed(pawn Killer, pawn Other, name damageType)
	killer = VC7;
	other = Z58;
	damageType = VC8;
	
	if(AGM.PawnFragged(VC7,Z58,VC8) == false)
	{
		//Super.Killed(VC7,Z58,VC8);
		BaseMutator.ScoreKill(VC7,Z58);
		
	   if ( bFreezeScores )
		  return;

		NotifyDeath = False;

		// Record the death no matter what, and reset the streak counter
		if (Other.bIsPlayer )
		{
			otherPlayer = DeusExPlayer(Other);
			Other.PlayerReplicationInfo.Deaths += 1;
			Other.PlayerReplicationInfo.Streak = 0;
			
			// Penalize the player that commits suicide by losing a kill, but don't take them below zero
			if ((Killer == Other) || (Killer == None))
			{
				if ( Other.PlayerReplicationInfo.Score > 0 )
				{
					if (( DeusExProjectile(otherPlayer.myProjKiller) != None ) && DeusExProjectile(otherPlayer.myProjKiller).bAggressiveExploded )
					{
						// Don't dock them if it nano exploded in their face
					}
					else
						Other.PlayerReplicationInfo.Score -= 1;
				}
			}
			NotifyDeath = True;
		}

	   //both players...
	   if ((Killer.bIsPlayer) && (Other.bIsPlayer))
	   {
			//Add to console log as well (with pri id) so that kick/kickban can work better
			log(Killer.PlayerReplicationInfo.PlayerName$"("$Killer.PlayerReplicationInfo.PlayerID$") killed "$Other.PlayerReplicationInfo.PlayerName $ otherPlayer.killProfile.methodStr);
			
			for (CurPawn = Level.PawnList; CurPawn != None; CurPawn = CurPawn.NextPawn)
			{
				if ((CurPawn.IsA('DeusExPlayer')) && (DeusExPlayer(CurPawn).bAdmin))
					DeusExPlayer(CurPawn).LocalLog(Killer.PlayerReplicationInfo.PlayerName$"|ccccccc("$Killer.PlayerReplicationInfo.PlayerID$") killed "$Other.PlayerReplicationInfo.PlayerName $"|ccccccc" $ otherPlayer.killProfile.methodStr);
			}

			if ( otherPlayer.killProfile.methodStr ~= "None" )
				BroadcastMessage(Killer.PlayerReplicationInfo.PlayerName$"|ccccccc killed "$Other.PlayerReplicationInfo.PlayerName$".",false,'DeathMessage');
			else
				BroadcastMessage(Killer.PlayerReplicationInfo.PlayerName$"|ccccccc killed "$Other.PlayerReplicationInfo.PlayerName $ "|ccccccc" $ otherPlayer.killProfile.methodStr, false, 'DeathMessage');

			if (Killer != Other)
			{
				// Penalize for killing your teammates
				if (ArePlayersAllied(DeusExPlayer(Other),DeusExPlayer(Killer)))
				{
					if ( Killer.PlayerReplicationInfo.Score > 0 )
						Killer.PlayerReplicationInfo.Score -= 1;
						
					DeusExPlayer(Killer).MultiplayerNotifyMsg( DeusExPlayer(Killer).MPMSG_KilledTeammate, 0, "" );
				}
				else
				{
					// Grant the kill to the killer, and increase his streak
					Killer.PlayerReplicationInfo.Score += 1;
					Killer.PlayerReplicationInfo.Streak += 1;

					Reward(Killer);

					// Check for victory conditions and end the match if need be
					if (CheckVictoryConditions(Killer, Other, otherPlayer.killProfile.methodStr))
					{
						bFreezeScores = True;
						NotifyDeath = False;
					}
				}
			}
			
			if ( NotifyDeath )
				HandleDeathNotification( Killer, Other );
	   }
	   else
	   {
			if (NotifyDeath)
				HandleDeathNotification( Killer, Other );

			Super.Killed(Killer,Other,damageType);
		}
	}
}

/**
  Disable haxy hax.
*/
function EditActor();

/**
  When a player restarts, reset their skin?
*/
function bool RestartPlayer( pawn aPlayer )	
{
  if(ANNAPlayer(aPlayer) != None && Manager != None && Manager.bResetSkin)
    ANNAPlayer(aPlayer).CheckSkin();

  return Super.RestartPlayer(aPlayer);
}


//these function have been ruined by MTL. Find some other way for GameMode to hook team switching...

/*
function bool ChangeTeam (Pawn Z61, int newTeam)
{
	if(AGM.PlayerChangedTeam(Z61, newTeam) == true)
		return false;
	
	Super.ChangeTeam(Z61, newTeam);
}

function bool ChangeTeam (Pawn Z61, int NewTeam)
{
	local Class<ANNAPlayer> Z62;
	local MTLPlayer Z63;

	if (  !ApproveTeam(NewTeam) )
	{
		NewTeam=128;
	}
	
	if ( NewTeam == 128 )
	{
		if ( PlayerPawn(Z61) != None )
		{
			NewTeam=PlayerPawn(Z61).PlayerReplicationInfo.Team;
		} else {
			return False;
		}
	}
	
	if (  !Super(DeusExMPGame).ChangeTeam(Z61,NewTeam) )
	{
		return False;
	}
	
	Z63=ANNAPlayer(Z61);
	
	if ( Z63 == None )
	{
		return False;
	}
	
	if ( NewTeam == 0 )
	{
		Z62=V1F;
	}
	else
	{
		if ( NewTeam == 1 )
		{
			Z62=V1E;
		}
		else
		{
			return False;
		}
	}
	
	return Class'v340'.V38(Z63,Z62);
}*/

function NavigationPoint FindPlayerStart(Pawn player, optional byte in_team, optional string incoming_name)
{
	local PlayerStart dest;
	local PlayerStart best_dest;
	local float best_score;
	local float cur_score;
	
	best_score = -1.0;
	best_dest = None;
	
	foreach AllActors(class 'PlayerStart', dest)
	{
		if(dest.bEnabled)
		{
			cur_score = EvaluatePlayerStartCDX(player, dest, in_team);
			if(cur_score > best_score)
			{
				best_score = cur_score;
				best_dest = dest;
			}
		}
	}
	
	return best_dest;
}

function float EvaluatePlayerStartCDX(Pawn player, PlayerStart nav_point, optional byte in_team)
{
	local DeusExPlayer other_player;
	local Pawn cur_pawn;
	local float dist;
	local float randomness;
	local float score;
	local float score_friendly;
	local float tmp_score;
	local float tmp_score_friendly;
	
	// Don't consider playerstarts for other game modes
	if(nav_point.bNonTeamOnlyStart)
		return -1.0;
	
	score = 3.0;
	score_friendly = 0.0;
	randomness = FRand() * 0.1;
	
	for(cur_pawn = Level.PawnList; cur_pawn != None; cur_pawn = cur_pawn.NextPawn)
	{
		other_player = DeusExPlayer(cur_pawn);
		
		if((other_player != None) && (other_player != Player) && (other_player.Health > 0) && (!other_player.IsInState('PlayerSpectating')))
		{
			// Get distance
			dist = VSize(other_player.Location - nav_point.Location);
			
			// Playerstarts that have even one player too close get disqualified
			if(dist < 100.0)
				return randomness;
			
			if(player.PlayerReplicationInfo.Team != other_player.PlayerReplicationInfo.Team)
			{
				// Only be influenced by enemies that are reasonably close
				if(dist < 1500.0)
				{
					tmp_score = 1.0 + (dist / 1500.0);
					
					if(tmp_score < score)
					{
						score = tmp_score;
					}
				}
			}
			else
			{
				// Friendlies who are moderately far away have an impact, but only a small impact
				if(dist < 3500.0)
				{
					tmp_score_friendly = 0.5 * (1.0 - (dist / 3500.0));
					
					if(tmp_score_friendly > score_friendly)
					{
						score_friendly = tmp_score_friendly;
					}
				}
			}
		}
	}
	
	return score + score_friendly + randomness;
}

DefaultProperties
{
	EnteredMessage="|ccccccc entered the game."
	LeftMessage="|ccccccc left the game."
	GameTypeName="CDX Team Deathmatch"
	bHasGameFinished=false
    GameName="Team Deathmatch"
    V1F=class'ANNAUNATCO'
    V1E=Class'ANNANSF'
    DefaultPlayerClass=Class'ANNAUNATCO'
    ScoreboardType=Class'ANNAScoreboard'
}
