//=============================================================================
// DeusExProjectile.
//=============================================================================
class DXProjectilev2 extends DeusExProjectile
	abstract;

var bool bHitMarkerGenerated;

function GrabProjectile(DeusExPlayer player)
{
	local Inventory item;
	local int collectType;
	local DeusExAmmo nonDispAmmo;

	if(spawnWeaponClass != None && spawnAmmoClass == None)
		if(spawnWeaponClass.Default.AmmoName != None)
			spawnAmmoClass = spawnWeaponClass.Default.AmmoName;

	if (player != None)
	{
		if(spawnWeaponClass != None)		// spawn the weapon
		{
			if(Player.FindInventoryType(spawnWeaponClass) == None)
			{
				item = Spawn(spawnWeaponClass);
				if(item != None)
				{
					DeusExWeapon(item).PickupAmmoCount = 1;
					collectType = 1;
				}
			}
		}
		if(spawnAmmoClass != None && collectType != 1)	// or spawn the ammo
		{
			item = Spawn(spawnAmmoClass);
			if (item != None)
			{
				Ammo(item).AmmoAmount = 1;
				collectType = 2;
			}
		}
		if (item != None)
		{
			player.FrobTarget = item;
			if(collectType == 1)
			{
				PlaySound(Sound'DeusExSounds.Generic.WeaponPickup');
				if(Level.Game.Difficulty > 1)
					Player.MakeNoise(0.1 * Level.Game.Difficulty);

				Item.Instigator = Player;
				Item.GotoState('Idle2');
				Item.bHeldItem = true;
				Item.bTossedOut = false;
				Weapon(Item).GiveAmmo(Player);
				Item.GiveTo(Player);
				if(DeusExAmmo(player.FindInventoryType(spawnAmmoClass)) != None)
				{
					DeusExAmmo(player.FindInventoryType(spawnAmmoClass)).AmmoAmount = 1;
					Player.ClientMessage(spawnWeaponClass.Default.PickupMessage @ spawnWeaponClass.Default.itemArticle @ spawnWeaponClass.Default.itemName, 'Pickup');
				}
				Destroy();	// destroy the projectile on the wall
			}				
			else if((collectType == 2) && spawnAmmoClass.Default.ItemName == "DEFAULT AMMO NAME - REPORT THIS AS A BUG")
			{
				PlaySound(Sound'DeusExSounds.Generic.WeaponPickup');
				if(Level.Game.Difficulty > 1)
					Player.MakeNoise(0.1 * Level.Game.Difficulty);

				nonDispAmmo=DeusExAmmo(player.FindInventoryType(spawnAmmoClass));
				if(nonDispAmmo != None)
				{
					nonDispAmmo.AddAmmo(1);
					if(spawnWeaponClass != None)
						Player.ClientMessage(spawnWeaponClass.Default.PickupMessage @ spawnWeaponClass.Default.itemArticle @ spawnWeaponClass.Default.itemName, 'Pickup');
				}

				if (item != None)
					item.Destroy();

				Destroy();
			}
			// check to see if we can pick up the new weapon/ammo
			else if (player.HandleItemPickup(item))
			{
				Destroy();	// destroy the projectile on the wall
				if(Level.NetMode != NM_Standalone)
				{
					if(item != None)
						item.Destroy();
				}
			}
			else
			{
				item.Destroy();			// destroy the weapon/ammo if it can't be picked up
			}
			player.FrobTarget = None;
		}
	}
}

simulated function SpawnEffects(Vector HitLocation, Vector HitNormal, Actor Other)
{
	local int i;
	local DeusExDecal mark;
	local Rockchip chip;

	//DEUS_EX AMSD Don't spawn these on the server.
	if ((Level.NetMode == NM_DedicatedServer) && (Role == ROLE_Authority))
		return;

	// don't draw damage art on destroyed movers
	if (DeusExMover(Other) != None)
		if (DeusExMover(Other).bDestroyed)
			ExplosionDecal = None;

	// draw the explosion decal here, not in Engine.Projectile
	if (ExplosionDecal != None)
	{
		mark = DeusExDecal(Spawn(ExplosionDecal, Self,, HitLocation, Rotator(HitNormal)));
		if (mark != None)
		{
			mark.DrawScale = FClamp(damage/30, 0.5, 3.0);
			mark.ReattachDecal();
		}
		ExplosionDecal = None;
	}

	if (bDebris)
	{
		for (i=0; i<Damage/5; i++)
		{
			if (FRand() < 0.8)
			{
				chip = spawn(class'Rockchip',,,HitLocation+HitNormal);
				//DEUS_EX AMSD In multiplayer, don't propagate these to 
				//other players (or from the listen server to clients).
				if (chip != None)
					chip.RemoteRole = ROLE_None;
			}
		}
	}
}

simulated function DrawExplosionEffects(vector HitLocation, vector HitNormal)
{
	local ShockRing ring;
	local SphereEffect sphere;
	local ExplosionLight light;
	local AnimatedSprite expeffect;
	local int i;

	// draw a pretty explosion
	light = Spawn(class'ExplosionLight',,, HitLocation);
	if (light != None)
		light.RemoteRole = ROLE_None;

	if (blastRadius < 128)
	{
		expeffect = Spawn(class'ExplosionSmall',,, HitLocation);
		light.size = 2;
	}
	else if (blastRadius < 256)
	{
		expeffect = Spawn(class'ExplosionMedium',,, HitLocation);
		light.size = 4;
	}
	else
	{
		expeffect = Spawn(class'ExplosionLarge',,, HitLocation);
		light.size = 8;
	}

	if (expeffect != None)
		expeffect.RemoteRole = ROLE_None;

	// draw a pretty shock ring
	// For nano defense we are doing something else.
	if ((!bAggressiveExploded) || (Level.NetMode == NM_Standalone))
	{
		ring = Spawn(class'ShockRing',,, HitLocation, rot(16384,0,0));
		if (ring != None)
		{
			ring.RemoteRole = ROLE_None;
			ring.size = blastRadius / 32.0;
		}
		ring = Spawn(class'ShockRing',,, HitLocation, rot(0,0,0));
		if (ring != None)
		{
			ring.RemoteRole = ROLE_None;
			ring.size = blastRadius / 32.0;
		}
		ring = Spawn(class'ShockRing',,, HitLocation, rot(0,16384,0));
		if (ring != None)
		{
			ring.RemoteRole = ROLE_None;
			ring.size = blastRadius / 32.0;
		}
	}
	else
	{
		for( i = 0; i < 3; i++ )
		{
			sphere = Spawn(class'SphereEffect',,, HitLocation, rot(0,0,0));
			if (sphere != None)
			{
				sphere.RemoteRole = ROLE_None;
				sphere.size = blastRadius / 32.0;
			}
		}
	}
}

simulated function Tick(float deltaTime)
{
	local float dist, size;
	local Rotator dir;
	local vector TargetLocation; 
	local vector vel;
	local vector NormalHeading;
	local vector NormalDesiredHeading;
	local float HeadingDiffDot;
	local vector zerovec;
	
	if(bStuck)
		return;

	if (VSize(LastSeenLoc) < 1)
		LastSeenLoc = Location + Normal(Vector(Rotation)) * 10000;

	if (Role == ROLE_Authority)
		bHasNetworkTarget = (Target != None);

	if (bTracking && ((Target != None) || ((Level.NetMode != NM_Standalone) && (bHasNetworkTarget))))
	{
		// check it's range
		if(Target == None)
			dist = MaxRange + 50; // Dirty, dirty hack to stop client log spam on death of a target.
		else
			dist = Abs(VSize(Target.Location - Location));
		if (dist > MaxRange)
		{
			// if we're out of range, lose the lock and quit tracking
			bTracking = False;
			Target = None;
			return;
		}
		else
		{
			// get the direction to the target
			if (Level.NetMode == NM_Standalone)
				TargetLocation = Target.Location;
			else
				TargetLocation = AcquireMPTargetLocation();
			if (Role == ROLE_Authority)
				NetworkTargetLoc = TargetLocation;
			LastSeenLoc = TargetLocation;
			dir = Rotator(TargetLocation - Location);
			dir.Roll = 0;

			if (Level.Netmode != NM_Standalone)
			{
				NormalHeading = Normal(Vector(Rotation));
				NormalDesiredHeading = Normal(TargetLocation - Location);
				HeadingDiffDot = NormalHeading Dot NormalDesiredHeading;
			}

			// set our new rotation
			bRotateToDesired = True;
			DesiredRotation = dir;

			// move us in the new direction that we are facing
			size = VSize(Velocity);
			vel = Normal(Vector(Rotation));

			if (Level.NetMode != NM_Standalone)
			{
				size = FMax(HeadingDiffDot,0.4) * Speed;
			}
			Velocity = vel * size;
		}
	}
	else
	{
		// make the rotation match the velocity direction
		SetRotation(Rotator(Velocity));
	}

	dist = Abs(VSize(initLoc - Location));

	if (dist > AccurateRange)		// start descent due to "gravity"
		Acceleration = Region.Zone.ZoneGravity / 2;

	if ((Role < ROLE_Authority) && (bAggressiveExploded))
		Explode(Location, vect(0,0,1));
}

simulated singular function BaseChange() //Cozmo: added this for things that stick to movers!
{
	Super.BaseChange();
	
	if(bStickToWall)
	{
		if ((base == None) && (Owner == None))
			SetPhysics(PHYS_Falling);
	}
}

defaultproperties
{
}
