//=============================================================================
// FirePuffEXFR:
//=============================================================================
//   Client-side particle effect for flames coming off of players.
//=============================================================================

class FirePuffEXFR extends Effects;

var vector finalLocation;
var vector initialOffset;
var float emberSpin;
var float emberMult;

auto simulated state Flying
{
	simulated function Tick(float delta_time)
	{
		local float intensity, f1, f2;
		local Rotator rot;
		
		intensity = LifeSpan / Default.LifeSpan;
		
		// Slow aging of embers a bit
		LifeSpan += emberMult * 0.35 * delta_time;
		
		// Opacity fading
		f1 = Sin(intensity * Pi * 0.5);
		f2 = Sin((1.0 - intensity) * Pi * 0.5);
		DrawScale = (OddsOfAppearing * 0.25 * (((1.0 - f1) * (1.0 - emberMult)) + 0.25)) * (1.0 - (emberMult * 0.6));
		ScaleGlow = 3.0 * (f2 - (1.0 - f1));
		
		// Ember spin rates
		rot.Yaw = emberSpin * 32000 * intensity;
		
		// Force location to the player
		finalLocation = Owner.Location;
		finalLocation.z += 42.0 + (emberMult * 29.0);
		SetLocation((intensity) * (Owner.Location + (initialOffset >> rot)) +
			(1.0 - intensity) * (finalLocation + ((initialOffset * (1.0 + (emberMult * 0.75)) * 1.25) >> rot)));
	}
}

function InitializeMe(int input)
{
	initialOffset = Owner.Location - Location;
	
	emberSpin = input - 2.5;
	if(emberSpin == -2.5)
	{
		emberSpin = 0.0;
		emberMult = 0.0;
	}
}

defaultproperties
{
	emberSpin=0.000000
	emberMult=1.000000
	LifeSpan=1.000000
	DrawType=DT_Sprite
	Style=STY_Translucent
	bUnlit=True
}