//=============================================================================
// FireballHeatEXFR:
//=============================================================================
//   Invisible projectile which serves as heat AOE, which stays attached to the
//   visible fireballs.
//=============================================================================

class FireballHeatEXFR extends DeusExProjectile;

simulated singular function Touch(Actor other_actor)
{
	local FireballEXFR attached_to;
	
	// Don't check anything unless we're the server
	if(Role < ROLE_Authority)
		return;
	
	attached_to = FireballEXFR(Owner);
	
	// Don't check for any collisions anymore if we're extinguishing
	if(attached_to.bExtinguishing)
		return;

	// Check the most common case first: Players
	if(other_actor.IsA('ANNAPlayer'))
	{
		if(other_actor == attached_to.Instigator)
		{
			if(attached_to.bBouncedOnce)
			{
				attached_to.ApplyHeatDamageToPlayer(ANNAPlayer(other_actor));
			}
		}
		else if(TeamDMGame(Level.Game) != None)
		{
			if(ANNAPlayer(other_actor).PlayerReplicationInfo.team != ANNAPlayer(attached_to.Instigator).PlayerReplicationInfo.team)
			{
				attached_to.ApplyHeatDamageToPlayer(ANNAPlayer(other_actor));
			}
		}
	}
	
	// Only other case we'll check for AOE heat: Grenades
	else if(other_actor.IsA('ThrownProjectile'))
	{
		// We can always destroy our own grenades
		if(ThrownProjectile(other_actor).Owner == ANNAPlayer(attached_to.Instigator))
		{
			ThrownProjectile(other_actor).TakeDamage(attached_to.Damage, attached_to.Instigator, attached_to.Location, Vect(0,0,0), 'Shot');
		}
		
		// In team games, only blow up enemy grenades
		else if(TeamDMGame(Level.Game) != None)
		{
			if(ThrownProjectile(other_actor).team != ANNAPlayer(attached_to.Instigator).PlayerReplicationInfo.team)
			{
				ThrownProjectile(other_actor).TakeDamage(attached_to.Damage, attached_to.Instigator, attached_to.Location, Vect(0,0,0), 'Shot');
			}
		}
		
		// This must be deathmatch, blow it up
		else
		{
			ThrownProjectile(other_actor).TakeDamage(attached_to.Damage, attached_to.Instigator, attached_to.Location, Vect(0,0,0), 'Shot');
		}
	}
}

defaultproperties
{
	bIgnoresNanoDefense=true
	speed=850.000000
	MaxSpeed=850.000000
	CollisionRadius=15.000000
	CollisionHeight=15.000000
	bCollideWorld=False
}