//=============================================================================
// Scope Window v1.15, by Lexichu (07/AUG/14)
// Changes:
// > Zoom in/out is now timed at 0.15 seconds regardless of FPS.
// > Fixed a bug where scoping may replicate incorrectly, causing FOV flicker.
//=============================================================================
class FixedDXScopeView expands DeusExScopeView;
//	config(DXWeapon2);

var float TimeEnd, FOVDiff, ChangeScale;
var bool bGZoomIn;
var config bool bFixedTimeZoom;

event InitWindow()
{
	Super(Window).InitWindow();
	
	// Get a pointer to the player
	player = DeusExPlayer(GetRootWindow().parentPawn);

	bTickEnabled = true;

	StyleChanged();
}

Function bool IsPlayerServer()
{
	local int Netmode;

	if(Player == None)
		return False;

	Netmode = Player.Level.Netmode;

	if(Netmode == 0)
		return True;
	if(Netmode == 1)
		return False;
	if(Netmode == 2)
		return Player.PlayerIsListenClient();
	if(Netmode == 3)
		return True;
}

// ----------------------------------------------------------------------
// Tick()
// ----------------------------------------------------------------------

event Tick(float deltaSeconds)
{
	local Crosshair        cross;
	local DeusExRootWindow dxRoot;

	if(IsPlayerServer())
	{
		if(DeusExWeapon(Player.InHand) != None || (DeusExPickup(Player.InHand) != None && bBinocs))
		{
			dxRoot = DeusExRootWindow(GetRootWindow());
			if (dxRoot != None)
			{
				cross = dxRoot.hud.cross;

				if (bActive)
					cross.SetCrosshair(false);
				else
					cross.SetCrosshair(player.bCrosshairVisible);
			}

			if(bFixedTimeZoom)
			{
				ChangeScale = (TimeEnd - Player.Level.TimeSeconds) * 4;
				if(ChangeScale < 0)
					ChangeScale = 0;

				if(bGZoomIn)
				{
					if(bBinocs)
					{
						Player.SetFOVAngle(20 + (FOVDiff * ChangeScale));
						Player.DesiredFOV = 20;
					}
					else
					{
						Player.SetFOVAngle(DeusExWeapon(Player.InHand).ScopeFOV + (FOVDiff * ChangeScale));	
						Player.DesiredFOV = DeusExWeapon(Player.InHand).ScopeFOV;
					}
				}
				else
				{
					if(bBinocs)
						Player.SetFOVAngle(Player.Default.DefaultFOV - ((Player.Default.DefaultFOV - 20) * ChangeScale));
					else
						Player.SetFOVAngle(Player.Default.DefaultFOV - ((Player.Default.DefaultFOV - DeusExWeapon(Player.InHand).ScopeFOV) * ChangeScale));
					Player.DesiredFOV = Player.Default.DefaultFOV;
				}
			}
		}
		else
		{
			Player.DesiredFOV = Player.Default.DefaultFOV;
			bGZoomIn = False;
		}
	}
}

Function FixedFOVChange(bool bZoomIn)
{
	local float TimeStart;

	TimeStart = Player.Level.TimeSeconds;
	TimeEnd = TimeStart + 0.15;
	bGZoomIn = bZoomIn;
	if(bZoomIn)
	{
		if(bBinocs)
			FOVDiff = Player.Default.DefaultFOV - 20;
		else
			FOVDiff = Player.Default.DefaultFOV - DeusExWeapon(Player.InHand).ScopeFOV;
	}
}

// ----------------------------------------------------------------------
// ActivateView()
// ----------------------------------------------------------------------

function ActivateView(int newFOV, bool bNewBinocs, bool bInstant)
{
	desiredFOV = newFOV;
	bBinocs = bNewBinocs;

	if (player != None)
	{
		if (bInstant)
			player.SetFOVAngle(newFOV);
		else
		{
			if(!bFixedTimeZoom)
				player.DesiredFOV = DesiredFOV;
			else
				FixedFOVChange(True);
		}
		bViewVisible = True;
		Show();
	}
}

// ----------------------------------------------------------------------
// DeactivateView()
// ----------------------------------------------------------------------

function DeactivateView()
{
	if (player != None && bViewVisible)
	{
		if(!bFixedTimeZoom)
			player.DesiredFOV = Player.Default.DefaultFOV;
		else
			FixedFOVChange(False);
		bViewVisible = False;
		HideView();
	}
}

// ----------------------------------------------------------------------
// HideView()
// ----------------------------------------------------------------------

function HideView()
{
	if (bViewVisible)
	{
		Hide();
		FixedFOVChange(False);
	}
}

// ----------------------------------------------------------------------
// ShowView()
// ----------------------------------------------------------------------

function ShowView()
{
	if (bViewVisible)
	{
		If(!bFixedTimeZoom)
			Player.SetFOVAngle(desiredFOV);
		else
			FixedFOVChange(True);

		Show();
	}
}
// ----------------------------------------------------------------------
// DrawWindow()
// ----------------------------------------------------------------------

event DrawWindow(GC gc)
{
	local float			fromX, toX;
	local float			fromY, toY;
	local float			scopeWidth, scopeHeight;
	local int			iX, iY;

	Super(Window).DrawWindow(gc);

	if(bViewVisible)
	{
		if (GetRootWindow().parentPawn != None)
		{
			if (player.IsInState('Dying'))
				return;
		}

		// Figure out where to put everything
		if (bBinocs)
			scopeWidth  = 512;
		else
			scopeWidth  = 256;

		scopeHeight = 256;

		fromX = (width-scopeWidth)/2;
		fromY = (height-scopeHeight)/2;
		toX   = fromX + scopeWidth;
		toY   = fromY + scopeHeight;

		// Draw the black borders
		gc.SetTileColorRGB(0, 0, 0);
		gc.SetStyle(DSTY_Normal);

		if ( Player.Level.NetMode == NM_Standalone || bBinocs )	// Only block out screen real estate in single player
		{
			gc.DrawPattern(0, 0, width, fromY, 0, 0, Texture'Solid');
			gc.DrawPattern(0, toY, width, fromY, 0, 0, Texture'Solid');
			gc.DrawPattern(0, fromY, fromX, scopeHeight, 0, 0, Texture'Solid');
			gc.DrawPattern(toX, fromY, fromX, scopeHeight, 0, 0, Texture'Solid');
		}
		// Draw the center scope bitmap
		// Use the Header Text color 

		if (bBinocs)
		{
			gc.SetStyle(DSTY_Modulated);
			gc.DrawTexture(fromX,       fromY, 256, scopeHeight, 0, 0, Texture'HUDBinocularView_1');
			gc.DrawTexture(fromX + 256, fromY, 256, scopeHeight, 0, 0, Texture'HUDBinocularView_2');

			gc.SetTileColor(colLines);
			gc.SetStyle(DSTY_Masked);
			gc.DrawTexture(fromX,       fromY, 256, scopeHeight, 0, 0, Texture'HUDBinocularCrosshair_1');
			gc.DrawTexture(fromX + 256, fromY, 256, scopeHeight, 0, 0, Texture'HUDBinocularCrosshair_2');
		}
		else
		{
			// Crosshairs - Use new scope in multiplayer, keep the old in single player
			if ( Player.Level.NetMode == NM_Standalone )
			{
				gc.SetStyle(DSTY_Modulated);
				gc.DrawTexture(fromX, fromY, scopeWidth, scopeHeight, 0, 0, Texture'HUDScopeView');
				gc.SetTileColor(colLines);
				gc.SetStyle(DSTY_Masked);
				gc.DrawTexture(fromX, fromY, scopeWidth, scopeHeight, 0, 0, Texture'HUDScopeCrosshair');
			}
			else
			{
				if (CDXWeaponRifle(Player.inHand) != None)
				{
					gc.SetStyle(DSTY_Modulated);
					gc.DrawTexture(fromX, fromY, scopeWidth, scopeHeight, 0, 0, Texture'HUDScopeView3');
				}
				else
				{
					gc.SetStyle(DSTY_Modulated);
					gc.DrawTexture(fromX, fromY, scopeWidth, scopeHeight, 0, 0, Texture'HUDScopeView2');
				}
			}
		}
	}
}

// ----------------------------------------------------------------------
// StyleChanged()
// ----------------------------------------------------------------------

event StyleChanged()
{
	local ColorTheme theme;

	theme = player.ThemeManager.GetCurrentHUDColorTheme();

	colLines = theme.GetColorFromName('HUDColor_HeaderText');
}

defaultproperties
{
}
