//=============================================================================
// LAM.
//=============================================================================
class ProjectileLAM extends ProjectileThrown
	config (DXWeapon2);

var float			mpBlastRadius;
var float			mpProxRadius;
var float			mpLAMDamage;
var float			mpFuselength;

var bool		bGlowingLAMs;
var config bool		bHardlightGlow;
var config bool		bZeroAugScalingLAM;
var config byte		glowBrightness;
var float	ZeroAugDamage, ZeroAugBlastRadius;


replication
{
	reliable if (Role == ROLE_Authority)
		bGlowingLAMs, bHardlightGlow, glowBrightness;
}

simulated function Tick(float deltaTime)
{
	local float blinkRate;

	Super.Tick(deltaTime);

	if (bDisabled)
	{
		Skin = Texture'BlackMaskTex';
		LightType = LT_None;
		return;
	}

	if(bProximityTriggered && !bProxActivated)
		blinkRate = 0.3;
	else
		blinkRate = 0.1;

	if ((Level.NetMode == NM_Standalone) || (Role == ROLE_Authority))
	{
		if (Abs((fuseLength - time)) % blinkRate > blinkRate * 0.5)
		{
			Skin = Texture'BlackMaskTex';
			LightType = LT_None;
		}
		else
		{
			Skin = Texture'LAM3rdTex1';
			LightType = LT_Steady;
		}
	}
}

simulated function PreBeginPlay()
{
	local DeusExMPGame agame;
	Super.PreBeginPlay();

	agame = DeusExMPGame(Level.Game);

	if(bGlowingLAMs && LightBrightness == 0)
	{
		LightBrightness = glowBrightness;
		if(bHardlightGlow)
			LightEffect = LE_NonIncidence;
		else
			LightEffect = LE_None;
	}

	if ( Level.NetMode != NM_Standalone )
	{
		proxRadius=mpProxRadius;
		if(aGame == None || !bZeroAugScalingLAM)
		{
			Damage = mpLAMDamage;
			blastRadius=mpBlastRadius;
		}
		else
		{
			if(aGame.AugsPerKill > 0 || aGame.InitialAugs > 0)
			{
				Damage = mpLAMDamage;
				blastRadius = mpBlastRadius;
			}
			else
			{
				Damage = ZeroAugDamage;
				blastRadius = ZeroAugBlastRadius;
				if(Damage < 1.000000)
					Damage = 120.000000;
				else if(Damage > 2048.000000)
					Damage = 2048.000000;

				if(BlastRadius < 16.000000)
					BlastRadius = 256.000000;
				else if(BlastRadius > 8192.000000)
					BlastRadius = 8192.000000;
			}
		}
		fuseLength=mpFuselength;
		bIgnoresNanoDefense=True;
	}
}

//Added by Cozmo. Don't let teammates shoot your nades!
simulated function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, name damageType)
{
	local ProjectileThrown pt;
	
	if(TeamDMGame(Level.Game) != None && Owner != None && instigatedBy != None && instigatedBy != Owner)
	{
		if(instigatedBy.PlayerReplicationInfo.Team == Pawn(Owner).PlayerReplicationInfo.Team)
		{
			if(damageType != 'Exploded')
				return;
		}
	}
	
	/* owner is set, but it still gives the kill to the original placer? (fun: I even tried spawning a second, identical nade belonging to instigatedBy, and even *that* somehow gave the kill to the wrong person...) will come back to this...
	if(instigatedBy.IsA('DeusExPlayer'))
		SetOwner(instigatedBy);
	*/
	
	Super.TakeDamage(Damage, instigatedBy, HitLocation, Momentum, damageType);
}

defaultproperties
{
bGlowingLAMs=false
     mpBlastRadius=512.000000
     mpProxRadius=128.000000
     mpLAMDamage=500.000000
     mpFuselength=1.500000
     glowBrightness=128
     ZeroAugDamage=500.000000
     ZeroAugBlastRadius=512.000000
     fuseLength=3.000000
     proxRadius=128.000000
     blastRadius=384.000000
     spawnWeaponClass=Class'CDX_140.WeaponLAM2' //this was commented out for some reason? hm
     ItemName="Lightweight Attack Munition (LAM)"
     speed=1000.000000
     MaxSpeed=1000.000000
     Damage=500.000000
     MomentumTransfer=50000
     ImpactSound=Sound'DeusExSounds.Weapons.LAMExplode'
     ExplosionDecal=Class'DeusEx.ScorchMark'
     LifeSpan=0.000000
     Mesh=LodMesh'DeusExItems.LAMPickup'
     CollisionRadius=4.300000
     CollisionHeight=3.800000
     LightRadius=1
     Mass=5.000000
     Buoyancy=2.000000
}
