//=============================================================================
// Cloud.
//=============================================================================
class ProjectileToxicGas extends DXProjectilev2;

var bool bFloating;
var float cloudRadius;
var float damageInterval;
var float damageTime;

auto simulated state Flying
{
	function HitWall(vector HitNormal, actor Wall)
	{
		// do nothing
		Velocity = vect(0,0,0);
	}
	function ProcessTouch (Actor Other, Vector HitLocation)
	{
		// do nothing
	}
}

event ZoneChange(ZoneInfo NewZone)
{
	Super.ZoneChange(NewZone);

	// clouds can't live underwater, so kill us quickly if we enter the water
	if ((NewZone.bWaterZone) && (LifeSpan > 2.0))
		LifeSpan = 2.0;
}

simulated function Tick(float deltaTime)
{
	local float value;
	local float sizeMult;
	local float NewDrawScale;
	local Actor A;
	local Pawn aPawn;

	time += deltaTime;
	damageTime += deltaTime;

	value = 1.0+time;
	if (MinDrawScale > 0)
		sizeMult = MaxDrawScale/MinDrawScale;
	else
		sizeMult = 1;

	if(Role == ROLE_Authority && damageTime >= damageInterval)
	{
		damageTime -= damageInterval;
		if ( Level.NetMode != NM_Standalone )
		{
			// Use PawnList for multiplayer
			apawn = Level.PawnList;
			while ( apawn != None )
			{
				if ( VSize(apawn.Location - Location) < cloudRadius )
				{
					apawn.TakeDamage( Damage, Instigator, aPawn.Location, vect(0,0,0), damageType );														  
				}
				apawn = apawn.nextPawn;
			}
		}
		else
		{
			foreach VisibleActors(class'Actor', A, cloudRadius)
			{
				if (A != Owner)
				{
					// be sure to damage the torso
					A.TakeDamage(Damage, Instigator, A.Location, vect(0,0,0), damageType);
				}
			}
		}
	}

	// DEUS_EX AMSD Update drawscale less often in mp, to reduce bandwidth hit.
	// Effect won't look quite as good for listen server client... but will otherwise
	// help tremendously (one gas grenade was 3k a sec in traffic).
	NewDrawScale = (-sizeMult/(value*value) + (sizeMult+1))*MinDrawScale;

	if (Level.Netmode == NM_Standalone)
	{
		DrawScale = NewDrawScale;
	}
	else if (Level.Netmode == NM_Client)
	{
		DrawScale = NewDrawScale;
	}
	else if (Level.Netmode == NM_DedicatedServer)
	{
		//Do nothing
	}
	else
	{
		//On a listen server, just start it full size.
		DrawScale = (-sizeMult/(50*50) + (sizeMult+1))*MinDrawScale;
	}

	if (Role == ROLE_Authority)	
		ScaleGlow = FClamp(LifeSpan*0.5, 0.0, 1.0);

	// make it swim around a bit at random
	if (bFloating)
	{
		Acceleration = VRand() * 15;
		Acceleration.Z = 0;
	}
}

defaultproperties
{
     cloudRadius=128.000000
     damageInterval=1.000000
     blastRadius=1.000000
     DamageType=TearGas
     AccurateRange=100
     maxRange=100
     maxDrawScale=2.000000
     bIgnoresNanoDefense=True
     ItemName="Gas Cloud"
     ItemArticle="a"
     MaxSpeed=300.000000
     Damage=8.000000
     MomentumTransfer=100
     LifeSpan=1.000000
     DrawType=DT_Sprite
     Style=STY_Translucent
     Texture=WetTexture'Effects.Smoke.Gas_Poison_A'
     DrawScale=0.010000
     bUnlit=True
     CollisionRadius=16.000000
     CollisionHeight=16.000000
}
