//=============================================================================
// WeaponEMPGrenade.
//=============================================================================
class WeaponEMPGrenade2 extends DeusExWeaponv2;

simulated function PreBeginPlay()
{
	Super.PreBeginPlay();

	// If this is a netgame, then override defaults
	if ( Level.NetMode != NM_StandAlone )
	{
		HitDamage = mpHitDamage;
		BaseAccuracy = mpBaseAccuracy;
		ReloadTime = mpReloadTime;
		AccurateRange = mpAccurateRange;
		MaxRange = mpMaxRange;
	}
}

function Projectile ProjectileFire(class<projectile> ProjClass, float ProjSpeed, bool bWarn)
{
	local Projectile proj;

	proj = Super.ProjectileFire(ProjClass, ProjSpeed, bWarn);

	if (proj != None)
		proj.PlayAnim('Open');
}

function PostBeginPlay()
{
	Super.PostBeginPlay();
	if(ScriptedPawn(Owner) == None)
		bWeaponStay=False;
}

function BecomePickup()
{
	Super.BecomePickup();
	if (Level.NetMode != NM_Standalone)
	{
		if (bTossedOut)
		{
			Lifespan = 0.0;
			RespawnTime = 0.0;
		}
	}
}

function BringUp()
{
	local DeusExPlayer dxp;

	Super.BringUp();
	PickupAmmoCount = 1;
}

auto state Pickup
{
	function Frob(Actor Other, Inventory frobWith)
	{
		local Ammo dxAmmo;
		local DeusExPlayer player;
		local bool bOtherGren;
		local inventory invex;

		for(invex = frobWith; invex != None; invex = invex.Inventory)
		{
			if(ClassIsChildOf(invex.Class, class'WeaponEMPGrenade'))
				bOtherGren = True;
		}

		player = DeusExPlayer(Other);
		if(player != None)
		{
			if(bOtherGren)
			{
				player.ClientMessage("You don't have enough room in your inventory to pick up" @ ItemArticle @ ItemName);
				return;
			}
			dxAmmo = Ammo(Player.findInventoryType(ammoName));
			if( (dxAmmo == None) || dxAmmo.AmmoAmount < dxAmmo.MaxAmmo )
			{
				player.ClientMessage(PickupMessage @ itemArticle @ itemName, 'Pickup');
				PlaySound (PickupSound);
				if ( Level.Game.Difficulty > 1 )
					Other.MakeNoise(0.1 * Level.Game.Difficulty);

				SpawnCopy(player);
			}
			else
			{
				player.ClientMessage("Cannot hold any more" @ ItemName);
			}
		}
	}
}

function inventory SpawnCopy( pawn Other )
{
	local inventory Copy;
	local Weapon newWeapon;
	local DeusExPlayer player;
	local bool bOccupied;

	player = DeusExPlayer(Other);

	if(player == None && Other != None)
		return Super.SpawnCopy(Other);

	bOccupied = !player.FindInventorySlot(self, True);

	if(Player.findInventoryType(Default.Class) == None)
	{
		if(bOccupied)
			return None;

		if( Level.Game.ShouldRespawn(self) && !bTossedOut )
		{
			Copy = spawn(Class,Other,,,rot(0,0,0));
			Copy.Tag           = Tag;
			Copy.Event         = Event;
			if ( !bWeaponStay )
				GotoState('Sleeping');
		}
		else
			Copy = self;

		Copy.RespawnTime = 0.0;
		Copy.bHeldItem = true;
		Copy.bTossedOut = false;

		// DEUS_EX AJY
		// Give weapon ammo before giving to player	
		Weapon(Copy).GiveAmmo(Other);
		Copy.GiveTo( Other );
		
		newWeapon = Weapon(Copy);
		newWeapon.Instigator = Other;
		newWeapon.SetSwitchPriority(Other);
		newWeapon.AmbientGlow = 0;
	}
	else
	{
		GiveAmmo(Other);
		if( !bTossedOut )
		{
			if ( !bWeaponStay )
				GotoState('Sleeping');
		}
		else
		{
			Destroy();
		}
	}
	return newWeapon;
}

simulated function Tick(float deltaTime)
{
	local DeusExPlayer dxp;
	local DeusExRootWindow root;

	Super.Tick(Deltatime);

	if(Level.Netmode == NM_Standalone || Role < ROLE_Authority)
	{
		dxp = DeusExPlayer(Owner);
		if(dxp != None)
			root = DeusExRootWindow(dxp.rootWindow);

		if(root != None)
		{
			if(beltPos != -1)
			{
				if(root.hud.belt.objects[beltpos] != None)
				{
					if(ammoType != None && ammoType.AmmoAmount > 1)
						root.hud.belt.objects[beltpos].itemText = root.hud.belt.objects[beltpos].CountLabel @ AmmoType.AmmoAmount;
					else
						root.hud.belt.objects[beltpos].itemText = "";
				}
			}
		}
	}
}

// ----------------------------------------------------------------------
// TestMPBeltSpot()
// Returns true if the suggested belt location is ok for the object in mp.
// ----------------------------------------------------------------------

simulated function bool TestMPBeltSpot(int BeltSpot)
{
	return (BeltSpot == 4);
}

defaultproperties
{
     H2HDelay=0.750000
     bDestroyOnEmpty=True
     bMPDestroyOnEmpty=True
     placeTime=0.454545
     LowAmmoWaterMark=2
     GoverningSkill=Class'DeusEx.SkillDemolition'
     EnemyEffective=ENMEFF_Robot
     EnviroEffective=ENVEFF_All
     Concealability=CONC_Visual
     ShotTime=0.727272
     reloadTime=0.100000
     HitDamage=0
     maxRange=4800
     AccurateRange=2400
     BaseAccuracy=1.000000
     bPenetrating=False
     StunDuration=60.000000
     bHasMuzzleFlash=False
     bHandToHand=True
     bUseAsDrawnWeapon=False
     AITimeLimit=3.500000
     AIFireDelay=5.000000
     bNeedToSetMPPickupAmmo=False
     mpReloadTime=0.100000
     mpBaseAccuracy=1.000000
     mpAccurateRange=2400
     mpMaxRange=2400
     AmmoName=Class'DeusEx.AmmoEMPGrenade'
     ReloadCount=1
     PickupAmmoCount=1
     FireOffset=(Y=10.000000,Z=20.000000)
     ProjectileClass=Class'CDX_140.ProjectileEMPGrenade'
     shakemag=50.000000
     SelectSound=Sound'DeusExSounds.Weapons.EMPGrenadeSelect'
     InventoryGroup=107
     ItemName="Electromagnetic Pulse (EMP) Grenade"
     ItemArticle="an"
     PlayerViewOffset=(X=24.000000,Y=-15.000000,Z=-19.000000)
     PlayerViewMesh=LodMesh'DeusExItems.EMPGrenade'
     PickupViewMesh=LodMesh'DeusExItems.EMPGrenadePickup'
     ThirdPersonMesh=LodMesh'DeusExItems.EMPGrenade3rd'
     Icon=Texture'DeusExUI.Icons.BeltIconEMPGrenade'
     largeIcon=Texture'DeusExUI.Icons.LargeIconEMPGrenade'
     largeIconWidth=31
     largeIconHeight=49
     Description="The EMP grenade creates a localized pulse that will temporarily disable all electronics within its area of effect, including cameras and security grids.|n|n<UNATCO OPS FILE NOTE JR134-VIOLET> While nanotech augmentations are largely unaffected by EMP, experiments have shown that it WILL cause the spontaneous dissipation of stored bioelectric energy. -- Jaime Reyes <END NOTE>"
     beltDescription="EMP GREN"
     Mesh=LodMesh'DeusExItems.EMPGrenadePickup'
     CollisionRadius=3.000000
     CollisionHeight=2.430000
     Mass=5.000000
     Buoyancy=2.000000
}
