//=============================================================================
// WeaponGasGrenade.
//=============================================================================
class WeaponGasGrenade2 extends DeusExWeaponv2;

simulated function PreBeginPlay()
{
	Super.PreBeginPlay();

	// If this is a netgame, then override defaults
	if ( Level.NetMode != NM_StandAlone )
	{
		HitDamage = mpHitDamage;
		BaseAccuracy = mpBaseAccuracy;
		ReloadTime = mpReloadTime;
		AccurateRange = mpAccurateRange;
		MaxRange = mpMaxRange;
	}
}

function PostBeginPlay()
{
	Super.PostBeginPlay();
	if(ScriptedPawn(Owner) == None)
		bWeaponStay=False;
}

function BecomePickup()
{
	Super.BecomePickup();
	if (Level.NetMode != NM_Standalone)
	{
		if (bTossedOut)
		{
			Lifespan = 0.0;
			RespawnTime = 0.0;
		}
	}
}

function BringUp()
{
	local DeusExPlayer dxp;

	Super.BringUp();
	PickupAmmoCount = 1;
}


auto state Pickup
{
	function Frob(Actor Other, Inventory frobWith)
	{
		local Ammo dxAmmo;
		local DeusExPlayer player;
		local bool bOtherGren;
		local inventory invex;

		for(invex = frobWith; invex != None; invex = invex.Inventory)
		{
			if(ClassIsChildOf(invex.Class, class'WeaponGasGrenade'))
				bOtherGren = True;
		}

		player = DeusExPlayer(Other);
		if(player != None)
		{
			if(bOtherGren)
			{
				player.ClientMessage("You don't have enough room in your inventory to pick up" @ ItemArticle @ ItemName);
				return;
			}
			dxAmmo = Ammo(Player.findInventoryType(ammoName));
			if( (dxAmmo == None) || dxAmmo.AmmoAmount < dxAmmo.MaxAmmo )
			{
				player.ClientMessage(PickupMessage @ itemArticle @ itemName, 'Pickup');
				PlaySound (PickupSound);
				if ( Level.Game.Difficulty > 1 )
					Other.MakeNoise(0.1 * Level.Game.Difficulty);

				SpawnCopy(player);
			}
			else
			{
				player.ClientMessage("Cannot hold any more" @ ItemName);
			}
		}
	}
}

function inventory SpawnCopy( pawn Other )
{
	local inventory Copy;
	local Weapon newWeapon;
	local DeusExPlayer player;
	local bool bOccupied;

	player = DeusExPlayer(Other);

	if(player == None && Other != None)
		return Super.SpawnCopy(Other);

	bOccupied = !player.FindInventorySlot(self, True);

	if(Player.findInventoryType(Default.Class) == None)
	{
		if(bOccupied)
			return None;

		if( Level.Game.ShouldRespawn(self) && !bTossedOut )
		{
			Copy = spawn(Class,Other,,,rot(0,0,0));
			Copy.Tag           = Tag;
			Copy.Event         = Event;
			if ( !bWeaponStay )
				GotoState('Sleeping');
		}
		else
			Copy = self;

		Copy.RespawnTime = 0.0;
		Copy.bHeldItem = true;
		Copy.bTossedOut = false;

		// DEUS_EX AJY
		// Give weapon ammo before giving to player	
		Weapon(Copy).GiveAmmo(Other);
		Copy.GiveTo( Other );
		
		newWeapon = Weapon(Copy);
		newWeapon.Instigator = Other;
		newWeapon.SetSwitchPriority(Other);
		newWeapon.AmbientGlow = 0;
	}
	else
	{
		GiveAmmo(Other);
		if( !bTossedOut )
		{
			if ( !bWeaponStay )
				GotoState('Sleeping');
		}
		else
		{
			Destroy();
		}
	}
	return newWeapon;
}

simulated function bool TestMPBeltSpot(int BeltSpot)
{
	return (BeltSpot == 5);
}

simulated function Tick(float deltaTime)
{
	local DeusExPlayer dxp;
	local DeusExRootWindow root;

	Super.Tick(Deltatime);

	if(Level.Netmode == NM_Standalone || Role < ROLE_Authority)
	{
		dxp = DeusExPlayer(Owner);
		if(dxp != None)
			root = DeusExRootWindow(dxp.rootWindow);

		if(root != None)
		{
			if(beltPos != -1)
			{
				if(root.hud.belt.objects[beltpos] != None)
				{
					if(ammoType != None && ammoType.AmmoAmount > 1)
						root.hud.belt.objects[beltpos].itemText = root.hud.belt.objects[beltpos].CountLabel @ AmmoType.AmmoAmount;
					else
						root.hud.belt.objects[beltpos].itemText = "";
				}
			}
		}
	}
}

defaultproperties
{
     H2HDelay=0.750000
     bDestroyOnEmpty=True
     bMPDestroyOnEmpty=True
     placeTime=0.854172
     LowAmmoWaterMark=2
     GoverningSkill=Class'DeusEx.SkillDemolition'
     EnemyEffective=ENMEFF_Organic
     EnviroEffective=ENVEFF_All
     Concealability=CONC_All
     ShotTime=0.833333
     reloadTime=0.100000
     HitDamage=0
     maxRange=4800
     AccurateRange=2400
     BaseAccuracy=1.000000
     bPenetrating=False
     StunDuration=60.000000
     bHasMuzzleFlash=False
     bHandToHand=True
     bUseAsDrawnWeapon=False
     AITimeLimit=4.000000
     AIFireDelay=20.000000
     bNeedToSetMPPickupAmmo=False
     mpReloadTime=0.100000
     mpHitDamage=2
     mpBaseAccuracy=1.000000
     mpAccurateRange=2400
     mpMaxRange=2400
     AmmoName=Class'DeusEx.AmmoGasGrenade'
     ReloadCount=1
     PickupAmmoCount=1
     FireOffset=(Y=10.000000,Z=20.000000)
     ProjectileClass=Class'CDX_140.ProjectileGasGrenade'
     shakemag=50.000000
     SelectSound=Sound'DeusExSounds.Weapons.GasGrenadeSelect'
     InventoryGroup=109
     ItemName="Gas Grenade"
     PlayerViewOffset=(Y=-13.000000,Z=-19.000000)
     PlayerViewMesh=LodMesh'DeusExItems.GasGrenade'
     PickupViewMesh=LodMesh'DeusExItems.GasGrenadePickup'
     ThirdPersonMesh=LodMesh'DeusExItems.GasGrenade3rd'
     Icon=Texture'DeusExUI.Icons.BeltIconGasGrenade'
     largeIcon=Texture'DeusExUI.Icons.LargeIconGasGrenade'
     largeIconWidth=23
     largeIconHeight=46
     Description="Upon detonation, the gas grenade releases a large amount of CS (a military-grade 'tear gas' agent) over its area of effect. CS will cause irritation to all exposed mucous membranes leading to temporary blindness and uncontrolled coughing. Like a LAM, gas grenades can be attached to any surface."
     beltDescription="GAS GREN"
     Mesh=LodMesh'DeusExItems.GasGrenadePickup'
     CollisionRadius=2.300000
     CollisionHeight=3.300000
     Mass=5.000000
     Buoyancy=2.000000
}
