//=============================================================================
// WeaponLAM.
//=============================================================================
class WeaponLAM2 extends DeusExWeaponv2;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	if(ScriptedPawn(Owner) == None)
		bWeaponStay=False;
}

function BecomePickup()
{
	Super.BecomePickup();
	if (Level.NetMode != NM_Standalone)
	{
		if (bTossedOut)
		{
			Lifespan = 0.0;
			RespawnTime = 0.0;
		}
	}
}

function BringUp()
{
	local DeusExPlayer dxp;

	Super.BringUp();
	PickupAmmoCount = 1;
}

simulated function Tick(float deltaTime)
{
	local DeusExPlayer dxp;
	local DeusExRootWindow root;

	Super.Tick(Deltatime);

	if(Level.Netmode == NM_Standalone || Role < ROLE_Authority)
	{
		dxp = DeusExPlayer(Owner);
		if(dxp != None)
			root = DeusExRootWindow(dxp.rootWindow);

		if(root != None)
		{
			if(beltPos != -1)
			{
				if(root.hud.belt.objects[beltpos] != None)
				{
					if(ammoType != None && ammoType.AmmoAmount > 1)
						root.hud.belt.objects[beltpos].itemText = root.hud.belt.objects[beltpos].CountLabel @ AmmoType.AmmoAmount;
					else
						root.hud.belt.objects[beltpos].itemText = "";
				}
			}
		}
	}
}

auto state Pickup
{
	function Frob(Actor Other, Inventory frobWith)
	{
		local Ammo dxAmmo;
		local DeusExPlayer player;
		local bool bOtherGren;
		local inventory invex;

		for(invex = frobWith; invex != None; invex = invex.Inventory)
		{
			if(ClassIsChildOf(invex.Class, class'WeaponLAM'))
				bOtherGren = True;
		}

		player = DeusExPlayer(Other);
		if(player != None)
		{
			if(bOtherGren)
			{
				player.ClientMessage("You don't have enough room in your inventory to pick up" @ ItemArticle @ ItemName);
				return;
			}
			dxAmmo = Ammo(Player.findInventoryType(ammoName));
			if( (dxAmmo == None) || dxAmmo.AmmoAmount < dxAmmo.MaxAmmo )
			{
				player.ClientMessage(PickupMessage @ itemArticle @ itemName, 'Pickup');
				PlaySound (PickupSound);
				if ( Level.Game.Difficulty > 1 )
					Other.MakeNoise(0.1 * Level.Game.Difficulty);

				SpawnCopy(player);
			}
			else
			{
				player.ClientMessage("Cannot hold any more" @ ItemName);
			}
		}
	}
}

function inventory SpawnCopy( pawn Other )
{
	local inventory Copy;
	local Weapon newWeapon;
	local DeusExPlayer player;
	local bool bOccupied;

	player = DeusExPlayer(Other);

	if(player == None && Other != None)
		return Super.SpawnCopy(Other);

	bOccupied = !player.FindInventorySlot(self, True);

	if(Player.findInventoryType(Default.Class) == None)
	{
		if(bOccupied)
			return None;

		if( Level.Game.ShouldRespawn(self) && !bTossedOut )
		{
			Copy = spawn(Class,Other,,,rot(0,0,0));
			Copy.Tag           = Tag;
			Copy.Event         = Event;
			if ( !bWeaponStay )
				GotoState('Sleeping');
		}
		else
			Copy = self;

		Copy.RespawnTime = 0.0;
		Copy.bHeldItem = true;
		Copy.bTossedOut = false;

		// DEUS_EX AJY
		// Give weapon ammo before giving to player	
		Weapon(Copy).GiveAmmo(Other);
		Copy.GiveTo( Other );
		
		newWeapon = Weapon(Copy);
		newWeapon.Instigator = Other;
		newWeapon.SetSwitchPriority(Other);
		newWeapon.AmbientGlow = 0;
	}
	else
	{
		GiveAmmo(Other);
		if( !bTossedOut )
		{
			if ( !bWeaponStay )
				GotoState('Sleeping');
		}
		else
		{
			Destroy();
		}
	}
	return newWeapon;
}

simulated function bool TestMPBeltSpot(int BeltSpot)
{
	return (BeltSpot == 6);
}

defaultproperties
{
     H2HDelay=0.750000
     bDestroyOnEmpty=True
     bMPDestroyOnEmpty=True
     placeTime=0.750000
     LowAmmoWaterMark=2
     GoverningSkill=Class'DeusEx.SkillDemolition'
     EnviroEffective=ENVEFF_All
     Concealability=CONC_All
     ShotTime=0.750000
     reloadTime=0.100000
     HitDamage=50
     maxRange=4800
     AccurateRange=2400
     BaseAccuracy=1.000000
     bHasMuzzleFlash=False
     bHandToHand=True
     bUseAsDrawnWeapon=False
     AITimeLimit=3.500000
     AIFireDelay=5.000000
     bNeedToSetMPPickupAmmo=False
     AmmoName=Class'DeusEx.AmmoLAM'
     ReloadCount=1
     PickupAmmoCount=1
     FireOffset=(Y=10.000000,Z=20.000000)
     ProjectileClass=Class'CDX_140.ProjectileLAM'
     shakemag=50.000000
     SelectSound=Sound'DeusExSounds.Weapons.LAMSelect'
     InventoryGroup=112
     ItemName="Lightweight Attack Munitions (LAM)"
     PlayerViewOffset=(X=24.000000,Y=-15.000000,Z=-17.000000)
     PlayerViewMesh=LodMesh'DeusExItems.LAM'
     PickupViewMesh=LodMesh'DeusExItems.LAMPickup'
     ThirdPersonMesh=LodMesh'DeusExItems.LAM3rd'
     Icon=Texture'DeusExUI.Icons.BeltIconLAM'
     largeIcon=Texture'DeusExUI.Icons.LargeIconLAM'
     largeIconWidth=35
     largeIconHeight=45
     Description="A multi-functional explosive with electronic priming system that can either be thrown or attached to any surface with its polyhesive backing and used as a proximity mine.|n|n<UNATCO OPS FILE NOTE SC093-BLUE> Disarming a proximity device should only be attempted with the proper demolitions training. Trust me on this. -- Sam Carter <END NOTE>"
     beltDescription="LAM"
     Mesh=LodMesh'DeusExItems.LAMPickup'
     CollisionRadius=3.800000
     CollisionHeight=3.500000
     Mass=5.000000
     Buoyancy=2.000000
}
